import { CommonModule } from "@angular/common";
import { Component, Input, NgModule } from "@angular/core";
import { IgxIconModule } from "../icon/icon.component";
export var Type;
(function (Type) {
    Type[Type["DEFAULT"] = 0] = "DEFAULT";
    Type[Type["INFO"] = 1] = "INFO";
    Type[Type["SUCCESS"] = 2] = "SUCCESS";
    Type[Type["WARNING"] = 3] = "WARNING";
    Type[Type["ERROR"] = 4] = "ERROR";
})(Type || (Type = {}));
export var Position;
(function (Position) {
    Position[Position["TOP_LEFT"] = 0] = "TOP_LEFT";
    Position[Position["TOP_RIGHT"] = 1] = "TOP_RIGHT";
    Position[Position["BOTTOM_LEFT"] = 2] = "BOTTOM_LEFT";
    Position[Position["BOTTOM_RIGHT"] = 3] = "BOTTOM_RIGHT";
})(Position || (Position = {}));
var IgxBadgeComponent = /** @class */ (function () {
    function IgxBadgeComponent() {
        this.typeEnum = Type;
        this.positionEnum = Position;
        this._type = "";
    }
    Object.defineProperty(IgxBadgeComponent.prototype, "type", {
        get: function () {
            return this._type === undefined ? "default" : this._type;
        },
        set: function (value) {
            var sizeType = this.typeEnum[value.toUpperCase()];
            this._type = sizeType === undefined ? "default" : value.toLowerCase();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxBadgeComponent.prototype, "position", {
        get: function () {
            return this._position === undefined ? "top-right" : this._position;
        },
        set: function (value) {
            var positionType = this.positionEnum[value.replace("-", "_").toUpperCase()];
            this._position = positionType === undefined ? "top-right" : value.toLowerCase();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxBadgeComponent.prototype, "value", {
        get: function () {
            return this._value === undefined ? "?" : this._value;
        },
        set: function (value) {
            this._value = value === undefined ? "-" : value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxBadgeComponent.prototype, "iconBdg", {
        get: function () {
            return this._iconBdg;
        },
        set: function (value) {
            this._iconBdg = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxBadgeComponent.prototype, "roleDescription", {
        get: function () {
            var message;
            // tslint:disable-next-line:prefer-conditional-expression
            if (this._iconBdg) {
                message = this._type + " type badge with icon type " + this._iconBdg;
            }
            else if (this._value) {
                message = this._type + " badge type with value " + this._value;
            }
            else {
                message = this._type + " badge type without value";
            }
            return message;
        },
        enumerable: true,
        configurable: true
    });
    IgxBadgeComponent.prototype.setClasses = function () {
        var classes = {};
        switch (this.typeEnum[this._type.toUpperCase()]) {
            case Type.DEFAULT:
                classes = {
                    "igx-badge__circle--default": true
                };
                break;
            case Type.INFO:
                classes = {
                    "igx-badge__circle--info": true
                };
                break;
            case Type.SUCCESS:
                classes = {
                    "igx-badge__circle--success": true
                };
                break;
            case Type.WARNING:
                classes = {
                    "igx-badge__circle--warning": true
                };
                break;
            case Type.ERROR:
                classes = {
                    "igx-badge__circle--error": true
                };
                break;
        }
        return classes;
    };
    IgxBadgeComponent.prototype.setPosition = function () {
        var className = {};
        switch (this.positionEnum[this.position.replace("-", "_").toUpperCase()]) {
            case Position.BOTTOM_LEFT:
                className = {
                    "igx-badge--bottom-left": true
                };
                break;
            case Position.BOTTOM_RIGHT:
                className = {
                    "igx-badge--bottom-right": true
                };
                break;
            case Position.TOP_LEFT:
                className = {
                    "igx-badge--top-left": true
                };
                break;
            case Position.TOP_RIGHT:
                className = {
                    "igx-badge--top-right": true
                };
                break;
        }
        return className;
    };
    IgxBadgeComponent.decorators = [
        { type: Component, args: [{
                    selector: "igx-badge",
                    template: "<div [ngClass]=\"setPosition()\" aria-label=\"badge\" role=\"img\"> \t<div class=\"igx-badge__circle\" [ngClass]=\"setClasses()\" [attr.aria-roledescription]=\"roleDescription\"> \t\t<span *ngIf=\"!iconBdg\" class=\"igx-badge__circle-value\">{{value}}</span> \t\t<igx-icon *ngIf=\"iconBdg\" fontSet=\"material\" [name]=\"iconBdg\"></igx-icon> \t\t<!-- <i *ngIf=\"iconBdg\" class=\"material-icons igx-badge__circle-value\" >{{iconBdg}}</i> --> \t</div> </div>"
                },] },
    ];
    /** @nocollapse */
    IgxBadgeComponent.ctorParameters = function () { return []; };
    IgxBadgeComponent.propDecorators = {
        "type": [{ type: Input },],
        "position": [{ type: Input },],
        "value": [{ type: Input },],
        "iconBdg": [{ type: Input, args: ["icon",] },],
    };
    return IgxBadgeComponent;
}());
export { IgxBadgeComponent };
var IgxBadgeModule = /** @class */ (function () {
    function IgxBadgeModule() {
    }
    IgxBadgeModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [IgxBadgeComponent],
                    exports: [IgxBadgeComponent],
                    imports: [CommonModule, IgxIconModule]
                },] },
    ];
    /** @nocollapse */
    IgxBadgeModule.ctorParameters = function () { return []; };
    return IgxBadgeModule;
}());
export { IgxBadgeModule };
//# sourceMappingURL=badge.component.js.map