import { ChangeDetectionStrategy, ChangeDetectorRef, Component, ElementRef, HostBinding, HostListener, Input, TemplateRef, ViewChild } from "@angular/core";
import { DataType } from "../data-operations/data-util";
import { IgxGridAPIService } from "./api.service";
import { IgxColumnComponent } from "./column.component";
var IgxGridCellComponent = /** @class */ (function () {
    function IgxGridCellComponent(gridAPI, cdr, element) {
        this.gridAPI = gridAPI;
        this.cdr = cdr;
        this.element = element;
        this.tabindex = 0;
        this.role = "gridcell";
        this.defaultCssClass = "igx-grid__td";
        this.isFocused = false;
        this.isSelected = false;
        this._inEditMode = false;
    }
    Object.defineProperty(IgxGridCellComponent.prototype, "formatter", {
        get: function () {
            return this.column.formatter;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridCellComponent.prototype, "context", {
        get: function () {
            return {
                $implicit: this.value,
                cell: this
            };
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridCellComponent.prototype, "template", {
        get: function () {
            if (this.inEditMode) {
                var inlineEditorTemplate = this.column.inlineEditorTemplate;
                return inlineEditorTemplate ? inlineEditorTemplate : this.inlineEditorTemplate;
            }
            if (this.cellTemplate) {
                return this.cellTemplate;
            }
            return this.defaultCellTemplate;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridCellComponent.prototype, "gridID", {
        get: function () {
            return this.row.gridID;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridCellComponent.prototype, "grid", {
        get: function () {
            return this.gridAPI.get(this.gridID);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridCellComponent.prototype, "rowIndex", {
        get: function () {
            return this.row.index;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridCellComponent.prototype, "columnIndex", {
        get: function () {
            return this.column.index;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridCellComponent.prototype, "nativeElement", {
        get: function () {
            return this.element.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridCellComponent.prototype, "inEditMode", {
        get: function () {
            return this._inEditMode;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridCellComponent.prototype, "readonly", {
        get: function () {
            return !this.column.editable;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridCellComponent.prototype, "describedby", {
        get: function () {
            return this.row.gridID + "_" + this.column.field;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridCellComponent.prototype, "styleClasses", {
        get: function () {
            return this.defaultCssClass + " " + this.column.cellClasses;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridCellComponent.prototype, "width", {
        get: function () {
            return this.column.width;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridCellComponent.prototype, "editModeCSS", {
        get: function () {
            return this._inEditMode;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridCellComponent.prototype, "focused", {
        get: function () {
            return this.isFocused || this.isSelected;
        },
        set: function (val) {
            this.isFocused = val;
            this.cdr.markForCheck();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridCellComponent.prototype, "applyNumberCSSClass", {
        get: function () {
            return this.column.dataType === DataType.Number;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridCellComponent.prototype, "selected", {
        get: function () {
            return this.isSelected;
        },
        set: function (val) {
            this.isSelected = val;
            this.cdr.markForCheck();
        },
        enumerable: true,
        configurable: true
    });
    IgxGridCellComponent.prototype.update = function (val) {
        this.grid.onEditDone.emit({ currentValue: this.value, newValue: val });
        this.value = val;
        this.gridAPI.update(this.gridID, this);
        this.cdr.markForCheck();
    };
    IgxGridCellComponent.prototype.onDoubleClick = function (event) {
        if (this.column.editable) {
            this._inEditMode = true;
            this.grid.cellInEditMode = this;
        }
    };
    IgxGridCellComponent.prototype.onFocus = function (event) {
        this.isFocused = true;
        this.isSelected = true;
        this.row.focused = true;
        if (this.grid.cellInEditMode && this.grid.cellInEditMode !== this) {
            this.grid.cellInEditMode._inEditMode = false;
            this.grid.cellInEditMode.cdr.markForCheck();
            this.grid.cellInEditMode = null;
        }
        this.grid.onSelection.emit(this);
        this.grid.cdr.detectChanges();
    };
    IgxGridCellComponent.prototype.onBlur = function (event) {
        this.isFocused = false;
        this.isSelected = false;
        this.row.focused = false;
    };
    IgxGridCellComponent.prototype.onKeydownArrowLeft = function () {
        var _this = this;
        var visibleColumns = this.grid.visibleColumns;
        var rowIndex = this.rowIndex;
        var columnIndex = this.columnIndex;
        var rv = visibleColumns.findIndex(function (col) { return col.index === columnIndex; });
        if (rv > 0) {
            columnIndex = visibleColumns[rv - 1].index;
            var target = this.gridAPI.get_cell_by_index(this.gridID, rowIndex, columnIndex);
            if (target) {
                target.nativeElement.focus();
                this.syncRows();
            }
            else {
                this.row.virtDirRow.scrollPrev();
                this.row.virtDirRow.onChunkLoaded.take(1).subscribe({
                    next: function (event) {
                        _this.row.cdr.detectChanges();
                        var currTarget = _this.gridAPI.get_cell_by_index(_this.gridID, rowIndex, columnIndex);
                        if (currTarget) {
                            currTarget.nativeElement.focus();
                        }
                        else {
                            _this.row.cells.first.nativeElement.focus();
                        }
                        setTimeout(function () {
                            _this.syncRows();
                        });
                    }
                });
            }
        }
    };
    IgxGridCellComponent.prototype.onKeydownCtrlArrowLeft = function () {
        var target = this.gridAPI.get_cell_by_index(this.gridID, this.rowIndex, this.row.cells.first.columnIndex);
        if (target) {
            target.nativeElement.focus();
        }
    };
    IgxGridCellComponent.prototype.onKeydownArrowRight = function () {
        var _this = this;
        var visibleColumns = this.grid.visibleColumns;
        var rowIndex = this.rowIndex;
        var columnIndex = this.columnIndex;
        var rv = visibleColumns.findIndex(function (col) { return col.index === columnIndex; });
        if (rv > -1 && rv < visibleColumns.length - 1) {
            columnIndex = visibleColumns[rv + 1].index;
            var target = this.gridAPI.get_cell_by_index(this.gridID, rowIndex, columnIndex);
            if (target) {
                target.nativeElement.focus();
                this.syncRows();
            }
            else {
                this.row.virtDirRow.scrollNext();
                this.row.virtDirRow.onChunkLoaded.take(1).subscribe({
                    next: function (event) {
                        _this.row.cdr.detectChanges();
                        var currTarget = _this.gridAPI.get_cell_by_index(_this.gridID, rowIndex, columnIndex);
                        if (currTarget) {
                            currTarget.nativeElement.focus();
                        }
                        else {
                            _this.row.cells.last.nativeElement.focus();
                        }
                        setTimeout(function () {
                            _this.syncRows();
                        });
                    }
                });
            }
        }
    };
    IgxGridCellComponent.prototype.onKeydownCtrlArrowRight = function () {
        var target = this.gridAPI.get_cell_by_index(this.gridID, this.rowIndex, this.row.cells.last.columnIndex);
        if (target) {
            target.nativeElement.focus();
        }
    };
    IgxGridCellComponent.prototype.onKeydownArrowUp = function () {
        var target = this.gridAPI.get_cell_by_index(this.gridID, this.rowIndex - 1, this.columnIndex);
        if (target) {
            target.nativeElement.focus();
        }
        else {
            this.row.grid.parentVirtDir.scrollPrev();
        }
    };
    IgxGridCellComponent.prototype.onKeydownArrowDown = function () {
        var target = this.gridAPI.get_cell_by_index(this.gridID, this.rowIndex + 1, this.columnIndex);
        if (target) {
            target.nativeElement.focus();
        }
        else {
            this.row.grid.parentVirtDir.scrollNext();
        }
    };
    IgxGridCellComponent.prototype.onKeydownEnterEditMode = function () {
        if (this.column.editable) {
            this._inEditMode = !this._inEditMode;
            this.grid.cellInEditMode = this;
        }
    };
    IgxGridCellComponent.prototype.onKeydownExitEditMode = function () {
        this._inEditMode = false;
    };
    IgxGridCellComponent.prototype.syncRows = function () {
        this.grid.cdr.detectChanges();
        var scrLeft = this.row.virtDirRow.dc.instance._viewContainer.element.nativeElement.scrollLeft;
        this.grid.headerContainer.dc.instance._viewContainer.element.nativeElement.style.left = (-scrLeft) + "px";
        var rows = this.row.grid.rowList.toArray();
        for (var _i = 0, rows_1 = rows; _i < rows_1.length; _i++) {
            var row = rows_1[_i];
            var elem = row.virtDirRow.dc.instance._viewContainer.element.nativeElement;
            elem.scrollLeft = scrLeft;
        }
    };
    IgxGridCellComponent.decorators = [
        { type: Component, args: [{
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    preserveWhitespaces: false,
                    selector: "igx-grid-cell",
                    template: "<ng-template #defaultCell>     {{ formatter ? formatter(value) : value }} </ng-template> <ng-template #inlineEditor let-cell=\"cell\">     <input igxInput [value]=\"value\" (keydown.enter)=\"cell.update($event.target.value)\"> </ng-template> <ng-container *ngTemplateOutlet=\"template; context: context\"> </ng-container>"
                },] },
    ];
    /** @nocollapse */
    IgxGridCellComponent.ctorParameters = function () { return [
        { type: IgxGridAPIService, },
        { type: ChangeDetectorRef, },
        { type: ElementRef, },
    ]; };
    IgxGridCellComponent.propDecorators = {
        "column": [{ type: Input },],
        "row": [{ type: Input },],
        "cellTemplate": [{ type: Input },],
        "value": [{ type: Input },],
        "tabindex": [{ type: HostBinding, args: ["attr.tabindex",] },],
        "role": [{ type: HostBinding, args: ["attr.role",] },],
        "readonly": [{ type: HostBinding, args: ["attr.aria-readonly",] },],
        "describedby": [{ type: HostBinding, args: ["attr.aria-describedby",] },],
        "styleClasses": [{ type: HostBinding, args: ["class",] },],
        "width": [{ type: HostBinding, args: ["style.min-width",] }, { type: HostBinding, args: ["style.flex-basis",] }, { type: HostBinding, args: ["class.igx-grid__td--fw",] },],
        "editModeCSS": [{ type: HostBinding, args: ["class.igx-grid__td--editing",] },],
        "focused": [{ type: HostBinding, args: ["attr.aria-selected",] }, { type: HostBinding, args: ["class.igx-grid__td--selected",] },],
        "applyNumberCSSClass": [{ type: HostBinding, args: ["class.igx-grid__td--number",] },],
        "defaultCellTemplate": [{ type: ViewChild, args: ["defaultCell", { read: TemplateRef },] },],
        "inlineEditorTemplate": [{ type: ViewChild, args: ["inlineEditor", { read: TemplateRef },] },],
        "onDoubleClick": [{ type: HostListener, args: ["dblclick", ["$event"],] },],
        "onFocus": [{ type: HostListener, args: ["focus", ["$event"],] },],
        "onBlur": [{ type: HostListener, args: ["blur", ["$event"],] },],
        "onKeydownArrowLeft": [{ type: HostListener, args: ["keydown.arrowleft",] },],
        "onKeydownCtrlArrowLeft": [{ type: HostListener, args: ["keydown.ctrl.arrowleft",] },],
        "onKeydownArrowRight": [{ type: HostListener, args: ["keydown.arrowright",] },],
        "onKeydownCtrlArrowRight": [{ type: HostListener, args: ["keydown.ctrl.arrowright",] },],
        "onKeydownArrowUp": [{ type: HostListener, args: ["keydown.arrowup",] },],
        "onKeydownArrowDown": [{ type: HostListener, args: ["keydown.arrowdown",] },],
        "onKeydownEnterEditMode": [{ type: HostListener, args: ["keydown.enter",] }, { type: HostListener, args: ["keydown.f2",] },],
        "onKeydownExitEditMode": [{ type: HostListener, args: ["keydown.escape",] },],
    };
    return IgxGridCellComponent;
}());
export { IgxGridCellComponent };
//# sourceMappingURL=cell.component.js.map