import { ChangeDetectionStrategy, ChangeDetectorRef, Component, ElementRef, forwardRef, HostBinding, HostListener, Input, QueryList, ViewChild, ViewChildren } from "@angular/core";
import { IgxForOfDirective } from "../directives/for-of/for_of.directive";
import { IgxGridAPIService } from "./api.service";
import { IgxGridCellComponent } from "./cell.component";
var IgxGridRowComponent = /** @class */ (function () {
    function IgxGridRowComponent(gridAPI, element, cdr) {
        this.gridAPI = gridAPI;
        this.element = element;
        this.cdr = cdr;
        this.tabindex = 0;
        this.role = "row";
        this.defaultCssClass = "igx-grid__tr";
        this.isFocused = false;
    }
    Object.defineProperty(IgxGridRowComponent.prototype, "styleClasses", {
        get: function () {
            return this.defaultCssClass + " " + (this.index % 2 ? this.grid.evenRowCSS : this.grid.oddRowCSS);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridRowComponent.prototype, "focused", {
        get: function () {
            return this.isFocused;
        },
        set: function (val) {
            this.isFocused = val;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridRowComponent.prototype, "columns", {
        get: function () {
            return this.grid.visibleColumns;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridRowComponent.prototype, "grid", {
        get: function () {
            return this.gridAPI.get(this.gridID);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridRowComponent.prototype, "nativeElement", {
        get: function () {
            return this.element.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    IgxGridRowComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.virtDirRow.onChunkLoaded.subscribe({
            next: function (event) {
                _this.grid.headerContainer.dc.instance._viewContainer.element.nativeElement.style.left = "0px";
            }
        });
    };
    IgxGridRowComponent.prototype.onFocus = function (event) {
        this.isFocused = true;
        // TODO: Emit selection event
    };
    IgxGridRowComponent.prototype.onBlur = function (event) {
        this.isFocused = false;
        // TODO: Emit de-selection event
    };
    IgxGridRowComponent.decorators = [
        { type: Component, args: [{
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    preserveWhitespaces: false,
                    selector: "igx-grid-row",
                    template: "<ng-template igxFor let-col [igxForOf]=\"columns\" [igxForScrollContainer]=\"grid.parentVirtDir\" let-colIndex=\"index\" [igxForScrollOrientation]=\"'horizontal'\" [igxForContainerSize]='grid.calcWidth' #igxDirRef>     <igx-grid-cell [column]=\"col\" [row]=\"this\" [style.min-width.px]=\"col.width\" [value]=\"rowData[col.field]\" [cellTemplate]=\"col.bodyTemplate\"></igx-grid-cell> </ng-template>"
                },] },
    ];
    /** @nocollapse */
    IgxGridRowComponent.ctorParameters = function () { return [
        { type: IgxGridAPIService, },
        { type: ElementRef, },
        { type: ChangeDetectorRef, },
    ]; };
    IgxGridRowComponent.propDecorators = {
        "rowData": [{ type: Input },],
        "index": [{ type: Input },],
        "gridID": [{ type: Input },],
        "virtDirRow": [{ type: ViewChild, args: ["igxDirRef", { read: IgxForOfDirective },] },],
        "cells": [{ type: ViewChildren, args: [forwardRef(function () { return IgxGridCellComponent; }), { read: IgxGridCellComponent },] },],
        "tabindex": [{ type: HostBinding, args: ["attr.tabindex",] },],
        "role": [{ type: HostBinding, args: ["attr.role",] },],
        "styleClasses": [{ type: HostBinding, args: ["class",] },],
        "focused": [{ type: HostBinding, args: ["attr.aria-selected",] }, { type: HostBinding, args: ["class.igx-grid__tr--selected",] },],
        "onFocus": [{ type: HostListener, args: ["focus", ["$event"],] },],
        "onBlur": [{ type: HostListener, args: ["blur", ["$event"],] },],
    };
    return IgxGridRowComponent;
}());
export { IgxGridRowComponent };
//# sourceMappingURL=row.component.js.map