import { CommonModule } from "@angular/common";
import { Component, ContentChildren, ElementRef, EventEmitter, forwardRef, HostBinding, Inject, Input, NgModule, Output, QueryList, Renderer2, ViewChild } from "@angular/core";
import { HammerGesturesManager } from "../core/touch";
import { IgxButtonModule } from "../directives/button/button.directive";
import { IgxRippleModule } from "../directives/ripple/ripple.directive";
export var IgxListPanState;
(function (IgxListPanState) {
    IgxListPanState[IgxListPanState["NONE"] = 0] = "NONE";
    IgxListPanState[IgxListPanState["LEFT"] = 1] = "LEFT";
    IgxListPanState[IgxListPanState["RIGHT"] = 2] = "RIGHT";
})(IgxListPanState || (IgxListPanState = {}));
// ====================== LIST ================================
// The `<igx-list>` directive is a list container for items and headers
var IgxListComponent = /** @class */ (function () {
    function IgxListComponent(element) {
        this.element = element;
        this.role = "list";
        this.allowLeftPanning = false;
        this.allowRightPanning = false;
        this.hasNoItemsTemplate = false;
        this.emptyListMessage = "No items";
        this.emptyListButtonText = "Add";
        this.emptyListButtonClick = new EventEmitter();
        this.onLeftPan = new EventEmitter();
        this.onRightPan = new EventEmitter();
        this.onPanStateChange = new EventEmitter();
        this._innerStyle = "igx-list";
    }
    Object.defineProperty(IgxListComponent.prototype, "innerStyle", {
        get: function () {
            return this._innerStyle;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxListComponent.prototype, "items", {
        get: function () {
            var items = [];
            if (this.children !== undefined) {
                for (var _i = 0, _a = this.children.toArray(); _i < _a.length; _i++) {
                    var child = _a[_i];
                    if (!child.isHeader) {
                        items.push(child);
                    }
                }
            }
            return items;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxListComponent.prototype, "headers", {
        get: function () {
            var headers = [];
            if (this.children !== undefined) {
                for (var _i = 0, _a = this.children.toArray(); _i < _a.length; _i++) {
                    var child = _a[_i];
                    if (child.isHeader) {
                        headers.push(child);
                    }
                }
            }
            return headers;
        },
        enumerable: true,
        configurable: true
    });
    IgxListComponent.prototype.onEmptyListButtonClicked = function (event) {
        this.emptyListButtonClick.emit({ list: this, event: event });
    };
    IgxListComponent.decorators = [
        { type: Component, args: [{
                    selector: "igx-list",
                    template: "<div class=\"{{innerStyle}}\">   <ng-content></ng-content>    <section *ngIf=\"children.length === 0 && !hasNoItemsTemplate\" class=\"igx-list__empty\">     <img class=\"image\" alt=\"{{ emptyListMessage }} image.\" src=\"{{ emptyListImage }}\" />      <article class=\"message\">       <p>{{ emptyListMessage }}</p>       <button igxButton=\"raised\" igxRipple=\"white\" (click)=\"onEmptyListButtonClicked($event)\">{{ emptyListButtonText }}</button>     </article>   </section>    <ng-content *ngIf=\"children.length === 0 && hasNoItemsTemplate\" select=\".igx-list__empty--custom\">   </ng-content>  </div>"
                },] },
    ];
    /** @nocollapse */
    IgxListComponent.ctorParameters = function () { return [
        { type: ElementRef, },
    ]; };
    IgxListComponent.propDecorators = {
        "children": [{ type: ContentChildren, args: [forwardRef(function () { return IgxListItemComponent; }),] },],
        "role": [{ type: HostBinding, args: ["attr.role",] },],
        "allowLeftPanning": [{ type: Input },],
        "allowRightPanning": [{ type: Input },],
        "hasNoItemsTemplate": [{ type: Input },],
        "emptyListImage": [{ type: Input },],
        "emptyListMessage": [{ type: Input },],
        "emptyListButtonText": [{ type: Input },],
        "emptyListButtonClick": [{ type: Output },],
        "onLeftPan": [{ type: Output },],
        "onRightPan": [{ type: Output },],
        "onPanStateChange": [{ type: Output },],
    };
    return IgxListComponent;
}());
export { IgxListComponent };
// ====================== ITEM ================================
// The `<igx-item>` directive is a container intended for row items in
// a `<igx-list>` container.
var IgxListItemComponent = /** @class */ (function () {
    function IgxListItemComponent(list, _renderer) {
        this.list = list;
        this._renderer = _renderer;
        this.hidden = false;
        this.isHeader = false;
        this._panState = IgxListPanState.NONE;
        this._FRACTION_OF_WIDTH_TO_TRIGGER_GRIP = 0.5;
        this._innerStyle = "";
        this._previousPanDeltaX = 0;
    }
    Object.defineProperty(IgxListItemComponent.prototype, "innerStyle", {
        get: function () {
            return this._innerStyle;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxListItemComponent.prototype, "panState", {
        get: function () {
            return this._panState;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxListItemComponent.prototype, "index", {
        get: function () {
            return this.list.children.toArray().indexOf(this);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxListItemComponent.prototype, "width", {
        get: function () {
            if (this.element && this.element.nativeElement) {
                return this.element.nativeElement.offsetWidth;
            }
            else {
                return 0;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxListItemComponent.prototype, "left", {
        get: function () {
            return this.element.nativeElement.offsetLeft;
        },
        set: function (value) {
            var val = value + "";
            if (val.indexOf("px") === -1) {
                val += "px";
            }
            this.element.nativeElement.style.left = val;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxListItemComponent.prototype, "maxLeft", {
        get: function () {
            return -this.width;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxListItemComponent.prototype, "maxRight", {
        get: function () {
            return this.width;
        },
        enumerable: true,
        configurable: true
    });
    IgxListItemComponent.prototype.ngOnInit = function () {
        if (this.isHeader) {
            this._innerStyle = "igx-list__header";
            this.role = "separator";
        }
        else {
            this._innerStyle = "igx-list__item";
            this.role = "listitem";
            this._addEventListeners();
            this.element.nativeElement.style.touchAction = "pan-y";
        }
        this.element.nativeElement.setAttribute("aria-label", this.element.nativeElement.textContent.trim());
    };
    IgxListItemComponent.prototype.ngOnDestroy = function () {
    };
    IgxListItemComponent.prototype._addEventListeners = function () {
        var _this = this;
        // Do not attach pan events if there is no options - no need to pan the item
        if (this._renderer && this.options && (this.list.allowLeftPanning || this.list.allowRightPanning)) {
            this._renderer.listen(this.element.nativeElement, "panstart", function (event) { _this.panStart(event); });
            this._renderer.listen(this.element.nativeElement, "panmove", function (event) { _this.panMove(event); });
            this._renderer.listen(this.element.nativeElement, "panend", function (event) { _this.panEnd(event); });
        }
    };
    IgxListItemComponent.prototype.panStart = function (ev) {
        this._previousPanDeltaX = 0;
    };
    IgxListItemComponent.prototype.panMove = function (ev) {
        var isPanningToLeft = this.left + ev.deltaX < this.left;
        if (isPanningToLeft) {
            this.left += ev.deltaX - this._previousPanDeltaX;
            if (this.list.allowRightPanning && !this.list.allowLeftPanning && this.left < 0) {
                this.left = 0;
            }
            else if (this.left < this.maxLeft) {
                this.left = this.maxLeft;
            }
        }
        else if (!isPanningToLeft) {
            this.left += ev.deltaX - this._previousPanDeltaX;
            if (this.list.allowLeftPanning && !this.list.allowRightPanning && this.left > 0) {
                this.left = 0;
            }
            else if (this.left > this.maxRight) {
                this.left = this.maxRight;
            }
        }
        this._previousPanDeltaX = ev.deltaX;
    };
    IgxListItemComponent.prototype.panEnd = function (ev) {
        var oldPanState = this._panState;
        this.performMagneticGrip();
        if (oldPanState !== this._panState) {
            this.list.onPanStateChange.emit({ oldState: oldPanState, newState: this._panState, item: this });
            if (this._panState === IgxListPanState.LEFT) {
                this.list.onLeftPan.emit(this);
            }
            else if (this._panState === IgxListPanState.RIGHT) {
                this.list.onRightPan.emit(this);
            }
        }
        this._previousPanDeltaX = 0;
    };
    IgxListItemComponent.prototype.performMagneticGrip = function () {
        var widthTriggeringGrip = this.width * this._FRACTION_OF_WIDTH_TO_TRIGGER_GRIP;
        if (this.left > 0) {
            if (this.left > widthTriggeringGrip) {
                this.left = this.maxRight;
                this._panState = IgxListPanState.RIGHT;
            }
            else {
                this.left = 0;
                this._panState = IgxListPanState.NONE;
            }
        }
        else {
            if (-this.left > widthTriggeringGrip) {
                this.left = this.maxLeft;
                this._panState = IgxListPanState.LEFT;
            }
            else {
                this.left = 0;
                this._panState = IgxListPanState.NONE;
            }
        }
    };
    IgxListItemComponent.decorators = [
        { type: Component, args: [{
                    providers: [HammerGesturesManager],
                    selector: "igx-list-item",
                    template: "<div #wrapper class=\"{{innerStyle}}\" [hidden]=\"hidden\">   <ng-content></ng-content> </div>"
                },] },
    ];
    /** @nocollapse */
    IgxListItemComponent.ctorParameters = function () { return [
        { type: IgxListComponent, decorators: [{ type: Inject, args: [forwardRef(function () { return IgxListComponent; }),] },] },
        { type: Renderer2, },
    ]; };
    IgxListItemComponent.propDecorators = {
        "element": [{ type: ViewChild, args: ["wrapper",] },],
        "role": [{ type: HostBinding, args: ["attr.role",] },],
        "isHeader": [{ type: Input },],
        "href": [{ type: Input },],
        "options": [{ type: Input },],
    };
    return IgxListItemComponent;
}());
export { IgxListItemComponent };
var IgxListModule = /** @class */ (function () {
    function IgxListModule() {
    }
    IgxListModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [IgxListComponent, IgxListItemComponent],
                    exports: [IgxListComponent, IgxListItemComponent],
                    imports: [CommonModule, IgxButtonModule, IgxRippleModule]
                },] },
    ];
    /** @nocollapse */
    IgxListModule.ctorParameters = function () { return []; };
    return IgxListModule;
}());
export { IgxListModule };
//# sourceMappingURL=list.component.js.map